@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.3 - a password manager for GNOME 2
+# Revelation 0.4.5 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: __init__.py 322 2005-03-31 17:43:29Z erikg $
+# $Id: __init__.py 412 2005-08-09 10:05:16Z erikg $
 #
 # Library initialization script
 #
@@ -26,7 +26,7 @@
 import crack
 import util
 import config
-import authmanager
+import gnomemisc
 import datahandler
 import io
 import entry
