@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.3 - a password manager for GNOME 2
+# Revelation 0.4.5 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: dialog.py 322 2005-03-31 17:43:29Z erikg $
+# $Id: dialog.py 419 2005-08-22 22:36:39Z erikg $
 #
 # Module with dialogs
 #
@@ -25,7 +25,7 @@
 
 import config, datahandler, entry, io, ui, util
 
-import gtk, gnome.ui, urllib
+import gobject, gtk, gnome.ui, urllib
 
 
 
@@ -33,6 +33,7 @@ RESPONSE_NEXT		= 10
 RESPONSE_PREVIOUS	= 11
 
 EVENT_FILTER		= None
+UNIQUE_DIALOGS		= {}
 
 
 
@@ -75,7 +76,7 @@ class Dialog(gtk.Dialog):
 	def __cb_keypress(self, widget, data):
 		"Callback for handling key presses"
 
-		# close the dialog on Escape
+		# close the dialog on escape
 		if data.keyval == 65307:
 			self.response(gtk.RESPONSE_CLOSE)
 
@@ -107,6 +108,66 @@ class Dialog(gtk.Dialog):
 
 
 
+class Popup(gtk.Window):
+	"Base class for popup (frameless) dialogs"
+
+	def __init__(self, widget = None):
+		gtk.Window.__init__(self)
+		self.set_decorated(False)
+
+		self.border = gtk.Frame()
+		self.border.set_shadow_type(gtk.SHADOW_OUT)
+		gtk.Window.add(self, self.border)
+
+		if widget != None:
+			self.add(widget)
+
+		self.connect("key-press-event", self.__cb_keypress)
+
+
+	def __cb_keypress(self, widget, data):
+		"Callback for key presses"
+
+		if data.keyval == 65307:
+			self.close()
+
+
+	def add(self, widget):
+		"Adds a widget to the window"
+
+		self.border.add(widget)
+
+
+	def close(self):
+		"Closes the dialog"
+
+		self.hide()
+		self.emit("closed")
+		self.destroy()
+
+
+	def realize(self):
+		"Realizes the popup and displays children"
+
+		gtk.Window.realize(self)
+
+		for child in self.get_children():
+			child.show_all()
+
+
+	def show(self, x = None, y = None):
+		"Show the dialog"
+
+		if x != None and y != None:
+			self.move(x, y)
+
+		self.show_all()
+
+
+gobject.signal_new("closed", Popup, gobject.SIGNAL_ACTION, gobject.TYPE_BOOLEAN, ())
+
+
+
 class Utility(Dialog):
 	"A utility dialog"
 
@@ -763,26 +824,24 @@ class PasswordSave(Password):
 class EntryEdit(Utility):
 	"A dialog for editing entries"
 
-	def __init__(self, parent, cfg, title, e = None):
-		if e == None:
-			stock = ui.STOCK_ADD
-
-		else:
-			stock = ui.STOCK_EDIT
-
+	def __init__(self, parent, title, e = None, cfg = None, clipboard = None):
 		Utility.__init__(
 			self, parent, title,
-			( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( stock, gtk.RESPONSE_OK ) )
+			(
+				( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ),
+				( e is None and ui.STOCK_NEW_ENTRY or ui.STOCK_EDIT, gtk.RESPONSE_OK )
+			)
 		)
 
 		self.config		= cfg
+		self.clipboard		= clipboard
 		self.entry_field	= {}
 		self.fielddata		= {}
 		self.widgetdata		= {}
 
 		# set up the ui
 		self.sect_meta		= self.add_section(title)
-		self.sect_fields	= None
+		self.sect_fields	= self.add_section("Account Data")
 
 		self.entry_name = ui.Entry()
 		self.entry_name.set_width_chars(50)
@@ -796,7 +855,7 @@ class EntryEdit(Utility):
 		self.dropdown = ui.EntryDropDown()
 		self.dropdown.connect("changed", lambda w: self.__setup_fieldsect(self.dropdown.get_active_type()().fields))
 		eventbox = ui.EventBox(self.dropdown)
-		self.tooltips.set_tip(eventbox, "The type of entry - folders can contain other entries")
+		self.tooltips.set_tip(eventbox, "The type of entry")
 		self.sect_meta.append_widget("Type", eventbox)
 
 		# populate the dialog with data
@@ -811,23 +870,17 @@ class EntryEdit(Utility):
 			self.fielddata[fieldtype] = fieldentry.get_text()
 
 		self.entry_field = {}
-
-		# create or remove field section as needed
-		if self.sect_fields is None and len(fields) > 0:
-			self.sect_fields = self.add_section("Account data")
-
-		elif self.sect_fields is not None and len(fields) > 0:
-			self.sect_fields.clear()
-
-		elif self.sect_fields is not None and len(fields) == 0:
-			self.sect_fields.destroy()
-			self.sect_fields = None
+		self.sect_fields.clear()
 
 		# generate field entries
 		for field in fields:
 
 			if self.widgetdata.has_key(type(field)):
 				userdata = self.widgetdata[type(field)]
+
+			elif field.datatype == entry.DATATYPE_PASSWORD:
+				userdata = self.clipboard
+
 			else:
 				userdata = None
 
@@ -848,14 +901,7 @@ class EntryEdit(Utility):
 
 
 		# show widgets
-		if self.sect_fields is not None:
-			self.sect_fields.show_all()
-
-
-	def allow_typechange(self, allow):
-		"Sets whether type changes are allowed"
-
-		self.dropdown.set_sensitive(allow)
+		self.sect_fields.show_all()
 
 
 	def get_entry(self):
@@ -952,6 +998,76 @@ class EntryRemove(Warning):
 
 
 
+class FolderEdit(Utility):
+	"Dialog for editing a folder"
+
+	def __init__(self, parent, title, e = None):
+		Utility.__init__(
+			self, parent, title,
+			(
+				( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ),
+				( e == None and ui.STOCK_NEW_FOLDER or ui.STOCK_EDIT, gtk.RESPONSE_OK )
+			)
+		)
+
+		# set up the ui
+		self.sect_folder	= self.add_section(title)
+
+		self.entry_name = ui.Entry()
+		self.entry_name.set_width_chars(25)
+		self.tooltips.set_tip(self.entry_name, "The name of the folder")
+		self.sect_folder.append_widget("Name", self.entry_name)
+
+		self.entry_desc = ui.Entry()
+		self.tooltips.set_tip(self.entry_desc, "A description of the folder")
+		self.sect_folder.append_widget("Description", self.entry_desc)
+
+		# populate the dialog with data
+		self.set_entry(e)
+
+
+	def get_entry(self):
+		"Generates an entry from the dialog contents"
+
+		e = entry.FolderEntry()
+		e.name = self.entry_name.get_text()
+		e.description = self.entry_desc.get_text()
+
+		return e
+
+
+	def run(self):
+		"Displays the dialog"
+
+		while 1:
+			self.show_all()
+
+			if Utility.run(self) == gtk.RESPONSE_OK:
+				e = self.get_entry()
+
+				if e.name == "":
+					Error(self, "Name not entered", "You must enter a name for the folder").run()
+					continue
+
+				self.destroy()
+				return e
+
+			else:
+				self.destroy()
+				raise CancelError
+
+
+	def set_entry(self, e):
+		"Sets an entry for the dialog"
+
+		if e is None:
+			return
+
+		self.entry_name.set_text(e.name)
+		self.entry_desc.set_text(e.description)
+
+
+
 ##### MISCELLANEOUS DIALOGS #####
 
 class About(gnome.ui.About):
@@ -962,7 +1078,7 @@ class About(gnome.ui.About):
 			self, config.APPNAME, config.VERSION, config.COPYRIGHT,
 			""""%s"\n\nRevelation is a password manager for the GNOME 2 desktop""" % config.RELNAME,
 			( config.AUTHOR, ), None, "",
-			gtk.icon_theme_get_default().load_icon("revelation", 48, 0)
+			parent.render_icon(ui.STOCK_REVELATION, gtk.ICON_SIZE_DIALOG)
 		)
 
 		if parent is not None:
@@ -1023,7 +1139,6 @@ class Find(Utility):
 
 		# account type dropdown
 		self.dropdown = ui.EntryDropDown()
-		self.dropdown.delete_item(0)
 		self.dropdown.insert_item(0, "Any", "gnome-stock-about")
 
 		eventbox = ui.EventBox(self.dropdown)
@@ -1079,7 +1194,7 @@ class PasswordChecker(Utility):
 	def __init__(self, parent):
 		Utility.__init__(
 			self, parent, "Password Checker",
-			( ( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE ), ( ui.STOCK_PASSWORD_CHECK, gtk.RESPONSE_OK ) )
+			( ( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE ), )
 		)
 
 		self.set_modal(False)
@@ -1088,49 +1203,53 @@ class PasswordChecker(Utility):
 		self.section = self.add_section("Password Checker")
 
 		self.entry = ui.Entry()
+		self.entry.set_width_chars(40)
+		self.entry.connect("changed", self.__cb_changed)
 		self.tooltips.set_tip(self.entry, "Enter a password to check")
 		self.section.append_widget("Password", self.entry)
 
-		self.result = ui.ImageLabel()
+		self.result = ui.ImageLabel("Enter a password to check", ui.STOCK_UNKNOWN)
+		self.result.set(0, 0.5, 0, 0)
 		self.tooltips.set_tip(self.result, "The result of the check")
-		self.section.append_widget(None, self.result, False)
+		self.section.append_widget(None, self.result)
 
 		self.connect("response", self.__cb_response)
-		self.response(gtk.RESPONSE_OK)
+
+
+	def __cb_changed(self, widget, data = None):
+		"Callback for entry changes"
+
+		password = self.entry.get_text()
+
+		try:
+			if len(password) == 0:
+				icon	= ui.STOCK_UNKNOWN
+				result	= "Enter a password to check"
+
+			else:
+				util.check_password(password)
+				icon	= ui.STOCK_PASSWORD_STRONG
+				result	= "The password is good"
+
+		except ValueError, result:
+			result	= str(result)
+			result = result.replace("simplistic/systematic", "systematic")
+			result = result.replace(" dictionary", "")
+
+			if result[:3] == "it ":
+				result = result[3:]
+
+			icon	= ui.STOCK_PASSWORD_WEAK
+			result = "The password " + result
+
+		self.result.set_text(result)
+		self.result.set_stock(icon, ui.ICON_SIZE_LABEL)
 
 
 	def __cb_response(self, widget, response):
-		"Callback for dialog responses"
+		"Callback for response"
 
-		if response == gtk.RESPONSE_OK:
-			password = self.entry.get_text()
-
-			try:
-				if len(password) == 0:
-					icon	= ui.STOCK_UNKNOWN
-					result	= "Enter a password to check"
-
-				else:
-					util.check_password(password)
-					icon	= ui.STOCK_PASSWORD_STRONG
-					result	= "The password is good"
-
-			except ValueError, result:
-				result	= str(result)
-				result = result.replace("simplistic/systematic", "systematic")
-				result = result.replace(" dictionary", "")
-
-				if result[:3] == "it ":
-					result = result[3:]
-
-				icon	= ui.STOCK_PASSWORD_WEAK
-				result = "The password " + result
-
-			self.result.set_text(result)
-			self.result.set_stock(icon, ui.ICON_SIZE_LABEL)
-
-		else:
-			self.destroy()
+		self.destroy()
 
 
 	def run(self):
@@ -1141,7 +1260,8 @@ class PasswordChecker(Utility):
 		if EVENT_FILTER != None:
 			self.window.add_filter(EVENT_FILTER)
 
-		# for some reason, gtk crashes on close-by-escape if we don't do this
+		# for some reason, gtk crashes on close-by-escape
+		# if we don't do this
 		self.get_button(0).grab_focus()
 		self.entry.grab_focus()
 
@@ -1219,12 +1339,13 @@ class Preferences(Utility):
 		self.__init_section_startup(self.page_files)
 		self.__init_section_filehandling(self.page_files)
 
-		self.page_behavior = self.notebook.create_page("Behavior")
-		self.__init_section_doubleclick(self.page_behavior)
-		self.__init_section_clipboard(self.page_behavior)
+		self.page_interface = self.notebook.create_page("Interface")
+		self.__init_section_doubleclick(self.page_interface)
+		self.__init_section_toolbar(self.page_interface)
 
 		self.page_password = self.notebook.create_page("Passwords")
 		self.__init_section_password_display(self.page_password)
+		self.__init_section_clipboard(self.page_password)
 		self.__init_section_pwgen(self.page_password)
 
 		self.page_gotocmd = self.notebook.create_page("Goto Commands")
@@ -1252,19 +1373,26 @@ class Preferences(Utility):
 		self.section_doubleclick = page.add_section("Doubleclick Action")
 
 		# radio-button for go to
-		self.radio_doubleclick_goto = ui.RadioButton(None, "Go to account on doubleclick, if possible")
+		self.radio_doubleclick_goto = ui.RadioButton(None, "Go to account, if possible")
 		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_goto, "goto")
 
 		self.tooltips.set_tip(self.radio_doubleclick_goto, "Go to the account (open in external application) on doubleclick, if command is set and required data filled in")
 		self.section_doubleclick.append_widget(None, self.radio_doubleclick_goto)
 
 		# radio-button for edit
-		self.radio_doubleclick_edit = ui.RadioButton(self.radio_doubleclick_goto, "Edit account on doubleclick")
+		self.radio_doubleclick_edit = ui.RadioButton(self.radio_doubleclick_goto, "Edit account")
 		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_edit, "edit")
 
 		self.tooltips.set_tip(self.radio_doubleclick_edit, "Edit the account on doubleclick")
 		self.section_doubleclick.append_widget(None, self.radio_doubleclick_edit)
 
+		# radio-button for copy
+		self.radio_doubleclick_copy = ui.RadioButton(self.radio_doubleclick_goto, "Copy password to clipboard")
+		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_copy, "copy")
+
+		self.tooltips.set_tip(self.radio_doubleclick_copy, "Copy the password to clipboard on doubleclick")
+		self.section_doubleclick.append_widget(None, self.radio_doubleclick_copy)
+
 
 	def __init_section_filehandling(self, page):
 		"Sets up the file section"
@@ -1383,6 +1511,47 @@ class Preferences(Utility):
 		self.section_startup.append_widget("File to open", eventbox)
 
 
+	def __init_section_toolbar(self, page):
+		"Sets up the toolbar section"
+
+		self.section_toolbar = page.add_section("Toolbar Style")
+
+		# radio-button for desktop default
+		self.radio_toolbar_desktop = ui.RadioButton(None, "Use desktop default")
+		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_desktop, "desktop")
+
+		self.tooltips.set_tip(self.radio_toolbar_desktop, "Show toolbar items with default style")
+		self.section_toolbar.append_widget(None, self.radio_toolbar_desktop)
+
+		# radio-button for icons and text
+		self.radio_toolbar_both = ui.RadioButton(self.radio_toolbar_desktop, "Show icons and text")
+		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_both, "both")
+
+		self.tooltips.set_tip(self.radio_toolbar_both, "Show toolbar items with both icons and text")
+		self.section_toolbar.append_widget(None, self.radio_toolbar_both)
+
+		# radio-button for icons and important text
+		self.radio_toolbar_bothhoriz = ui.RadioButton(self.radio_toolbar_desktop, "Show icons and important text")
+		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_bothhoriz, "both-horiz")
+
+		self.tooltips.set_tip(self.radio_toolbar_bothhoriz, "Show toolbar items with text beside important icons")
+		self.section_toolbar.append_widget(None, self.radio_toolbar_bothhoriz)
+
+		# radio-button for icons only
+		self.radio_toolbar_icons = ui.RadioButton(self.radio_toolbar_desktop, "Show icons only")
+		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_icons, "icons")
+
+		self.tooltips.set_tip(self.radio_toolbar_icons, "Show toolbar items with icons only")
+		self.section_toolbar.append_widget(None, self.radio_toolbar_icons)
+
+		# radio-button for text only
+		self.radio_toolbar_text = ui.RadioButton(self.radio_toolbar_desktop, "Show text only")
+		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_text, "text")
+
+		self.tooltips.set_tip(self.radio_toolbar_text, "Show toolbar items with text only")
+		self.section_toolbar.append_widget(None, self.radio_toolbar_text)
+
+
 	def run(self):
 		"Runs the preference dialog"
 
@@ -1395,3 +1564,66 @@ class Preferences(Utility):
 		self.get_button(0).grab_focus()
 		self.notebook.grab_focus()
 
+
+
+##### FUNCTIONS #####
+
+def create_unique(dialog, *args):
+	"Creates a unique dialog"
+
+	if present_unique(dialog) == True:
+		return get_unique(dialog)
+
+	else:
+		UNIQUE_DIALOGS[dialog] = dialog(*args)
+		UNIQUE_DIALOGS[dialog].connect("destroy", lambda w: remove_unique(dialog))
+
+		return UNIQUE_DIALOGS[dialog]
+
+
+def get_unique(dialog):
+	"Returns a unique dialog"
+
+	if unique_exists(dialog) == True:
+		return UNIQUE_DIALOGS[dialog]
+
+	else:
+		return None
+
+
+def present_unique(dialog):
+	"Presents a unique dialog, if it exists"
+
+	if unique_exists(dialog) == True:
+		get_unique(dialog).present()
+
+		return True
+
+	else:
+		return False
+
+
+def remove_unique(dialog):
+	"Removes a unique dialog"
+
+	if unique_exists(dialog):
+		UNIQUE_DIALOGS[dialog] = None
+
+
+def run_unique(dialog, *args):
+	"Runs a unique dialog"
+
+	if present_unique(dialog) == True:
+		return None
+
+	else:
+		d = create_unique(dialog, *args)
+
+		return d.run()
+
+
+def unique_exists(dialog):
+	"Checks if a unique dialog exists"
+
+	return UNIQUE_DIALOGS.has_key(dialog) == True and UNIQUE_DIALOGS[dialog] != None
+
