@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.3 - a password manager for GNOME 2
+# Revelation 0.4.5 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: io.py 322 2005-03-31 17:43:29Z erikg $
+# $Id: io.py 412 2005-08-09 10:05:16Z erikg $
 #
 # Module for IO-related functionality
 #
@@ -25,7 +25,13 @@
 
 import datahandler
 
-import gobject, gnome.vfs, os.path, re
+import gobject, os.path, re
+
+try:
+	import gnomevfs
+
+except ImportError:
+	import gnome.vfs as gnomevfs
 
 
 
@@ -50,7 +56,7 @@ class DataFile(gobject.GObject):
 	def __cb_monitor(self, monitor_uri, info_uri, event, data = None):
 		"Callback for file monitoring"
 
-		if event == gnome.vfs.MONITOR_EVENT_CHANGED:
+		if event == gnomevfs.MONITOR_EVENT_CHANGED:
 			self.emit("content-changed", self.get_file())
 
 
@@ -107,7 +113,7 @@ class DataFile(gobject.GObject):
 
 		self.__handler.check(data)
 
-		if self.__handler.encryption == True and password is None:
+		if self.__handler.encryption == True and password is None and pwgetter != None:
 			password = pwgetter()
 
 		entrystore = self.__handler.import_data(data, password)
@@ -132,7 +138,7 @@ class DataFile(gobject.GObject):
 	def set_file(self, file):
 		"Sets the current file"
 
-		uri = file is not None and gnome.vfs.URI(file_normpath(file)) or None
+		uri = file is not None and gnomevfs.URI(file_normpath(file)) or None
 
 		if self.__uri != uri:
 			self.__uri = uri
@@ -165,7 +171,7 @@ def file_exists(file):
 	if file is None:
 		return False
 
-	return gnome.vfs.exists(file)
+	return gnomevfs.exists(file)
 
 
 def file_is_local(file):
@@ -174,7 +180,7 @@ def file_is_local(file):
 	if file is None:
 		return False
 
-	uri = gnome.vfs.URI(file)
+	uri = gnomevfs.URI(file)
 
 	return uri.is_local
 
@@ -183,16 +189,16 @@ def file_monitor(file, callback):
 	"Starts monitoring a file"
 
 	try:
-		return gnome.vfs.monitor_add(file_normpath(file), gnome.vfs.MONITOR_FILE, callback)
+		return gnomevfs.monitor_add(file_normpath(file), gnomevfs.MONITOR_FILE, callback)
 
-	except gnome.vfs.NotSupportedError:
+	except gnomevfs.NotSupportedError:
 		return None
 
 
 def file_monitor_cancel(handle):
 	"Cancels file monitoring"
 
-	gnome.vfs.monitor_cancel(handle)
+	gnomevfs.monitor_cancel(handle)
 
 
 def file_normpath(file):
@@ -206,7 +212,7 @@ def file_normpath(file):
 	if not re.match("^[a-zA-Z]+://", file) and file[0] != "/":
 		file = os.path.abspath(file)
 
-	return re.sub("^file:/{,2}", "", str(gnome.vfs.URI(file)))
+	return re.sub("^file:/{,2}", "", str(gnomevfs.URI(file)))
 
 
 def file_read(file):
@@ -216,9 +222,9 @@ def file_read(file):
 		if file is None:
 			raise IOError
 
-		return gnome.vfs.read_entire_file(file)
+		return gnomevfs.read_entire_file(file)
 
-	except ( gnome.vfs.AccessDeniedError, gnome.vfs.NotFoundError, gnome.vfs.EOFError ):
+	except gnomevfs.Error:
 		raise IOError
 
 
@@ -233,14 +239,14 @@ def file_write(file, data):
 			data = ""
 
 		if file_exists(file) == True:
-			f = gnome.vfs.open(file, gnome.vfs.OPEN_WRITE)
+			f = gnomevfs.open(file, gnomevfs.OPEN_WRITE)
 
 		else:
-			f = gnome.vfs.create(file, gnome.vfs.OPEN_WRITE)
+			f = gnomevfs.create(file, gnomevfs.OPEN_WRITE)
 
 		f.write(data)
 		f.close()
 
-	except gnome.vfs.AccessDeniedError:
+	except gnomevfs.Error:
 		raise IOError
 
