@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.3 - a password manager for GNOME 2
+# Revelation 0.4.5 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: ui.py 322 2005-03-31 17:43:29Z erikg $
+# $Id: ui.py 412 2005-08-09 10:05:16Z erikg $
 #
 # Module for UI functionality
 #
@@ -25,19 +25,21 @@
 
 import config, data, dialog, entry, io, util
 
-import bonobo.ui, gobject, gtk, gnome.ui, os, pango, pwd, time, xml.dom.minidom
+import bonobo.ui, gobject, gtk, gtk.gdk, gnome.ui, os, pango, pwd, time, xml.dom.minidom
 from xml.parsers.expat import ExpatError
 
 
-STOCK_ADD			= "revelation-add"
 STOCK_CONTINUE			= "revelation-continue"
 STOCK_DISCARD			= "revelation-discard"
 STOCK_EDIT			= "revelation-edit"
 STOCK_EXPORT			= "revelation-export"
+STOCK_FOLDER			= "revelation-folder"
 STOCK_GENERATE			= "revelation-generate"
 STOCK_IMPORT			= "revelation-import"
 STOCK_GOTO			= "revelation-goto"
 STOCK_LOCK			= "revelation-lock"
+STOCK_NEW_ENTRY			= "revelation-new-entry"
+STOCK_NEW_FOLDER		= "revelation-new-folder"
 STOCK_NEXT			= "revelation-next"
 STOCK_OVERWRITE			= "revelation-overwrite"
 STOCK_PASSWORD_CHANGE		= "revelation-password-change"
@@ -51,8 +53,8 @@ STOCK_UNKNOWN			= "revelation-unknown"
 STOCK_UNLOCK			= "revelation-unlock"
 
 
-STOCK_ENTRY_FOLDER		= "revelation-folder"
-STOCK_ENTRY_FOLDER_OPEN		= "revelation-folder-open"
+STOCK_ENTRY_FOLDER		= "revelation-account-folder"
+STOCK_ENTRY_FOLDER_OPEN		= "revelation-account-folder-open"
 STOCK_ENTRY_CREDITCARD		= "revelation-account-creditcard"
 STOCK_ENTRY_CRYPTOKEY		= "revelation-account-cryptokey"
 STOCK_ENTRY_DATABASE		= "revelation-account-database"
@@ -65,14 +67,19 @@ STOCK_ENTRY_SHELL		= "revelation-account-shell"
 STOCK_ENTRY_WEBSITE		= "revelation-account-website"
 
 STOCK_REVELATION		= "revelation-revelation"
+STOCK_REVELATION_LOCKED		= "revelation-revelation-locked"
 
 
+ICON_SIZE_APPLET		= gtk.icon_size_from_name("revelation-applet")
 ICON_SIZE_DATAVIEW		= gtk.icon_size_from_name("revelation-dataview")
 ICON_SIZE_DROPDOWN		= gtk.icon_size_from_name("revelation-dropdown")
 ICON_SIZE_LABEL			= gtk.icon_size_from_name("revelation-label")
 ICON_SIZE_LOGO			= gtk.icon_size_from_name("revelation-logo")
 ICON_SIZE_TREEVIEW		= gtk.icon_size_from_name("revelation-treeview")
 
+if ICON_SIZE_APPLET == gtk.ICON_SIZE_INVALID:
+	ICON_SIZE_APPLET	= gtk.icon_size_register("revelation-applet", 24, 24)
+
 if ICON_SIZE_DATAVIEW == gtk.ICON_SIZE_INVALID:
 	ICON_SIZE_DATAVIEW	= gtk.icon_size_register("revelation-dataview", 24, 24)
 
@@ -140,14 +147,14 @@ def config_bind(cfg, key, widget, data = None):
 	return id
 
 
-def generate_field_display_widget(field, cfg = None):
+def generate_field_display_widget(field, cfg = None, userdata = None):
 	"Generates a widget for displaying a field value"
 
 	if field.datatype == entry.DATATYPE_EMAIL:
 		widget = LinkButton("mailto:%s" % field.value, util.escape_markup(field.value))
 
 	elif field.datatype == entry.DATATYPE_PASSWORD:
-		widget = PasswordLabel(util.escape_markup(field.value), cfg)
+		widget = PasswordLabel(util.escape_markup(field.value), cfg, userdata)
 
 	elif field.datatype == entry.DATATYPE_URL:
 		widget = LinkButton(field.value, util.escape_markup(field.value))
@@ -163,7 +170,7 @@ def generate_field_edit_widget(field, cfg = None, userdata = None):
 	"Generates a widget for editing a field"
 
 	if type(field) == entry.PasswordField:
-		widget = PasswordEntryGenerate(cfg)
+		widget = PasswordEntryGenerate(None, cfg, userdata)
 
 	elif type(field) == entry.UsernameField:
 		widget = ComboBoxEntry(userdata)
@@ -172,7 +179,7 @@ def generate_field_edit_widget(field, cfg = None, userdata = None):
 		widget = FileEntry()
 
 	elif field.datatype == entry.DATATYPE_PASSWORD:
-		widget = PasswordEntry(cfg)
+		widget = PasswordEntry(None, cfg, userdata)
 
 	else:
 		widget = Entry()
@@ -199,6 +206,20 @@ class HBox(gtk.HBox):
 
 
 
+class HButtonBox(gtk.HButtonBox):
+	"A horizontal button box"
+
+	def __init__(self, *args):
+		gtk.HButtonBox.__init__(self)
+
+		self.set_layout(gtk.BUTTONBOX_SPREAD)
+		self.set_spacing(12)
+
+		for button in args:
+			self.pack_start(button)
+
+
+
 class VBox(gtk.VBox):
 	"A vertical container"
 
@@ -326,7 +347,7 @@ class InputSection(VBox):
 		self.pack_start(row, False, False)
 
 		if self.title is not None and indent == True:
-			row.pack_start(Label("   "), False, False)
+			row.pack_start(Label(""), False, False)
 
 		if title is not None:
 			label = Label("%s:" % util.escape_markup(title))
@@ -374,7 +395,7 @@ class Image(gtk.Image):
 class ImageLabel(gtk.Alignment):
 	"A label with an image"
 
-	def __init__(self, text = None, stock = None, size = None):
+	def __init__(self, text = None, stock = None, size = ICON_SIZE_LABEL):
 		gtk.Alignment.__init__(self, 0.5, 0.5, 0, 0)
 
 		self.hbox = HBox()
@@ -438,41 +459,79 @@ class Label(gtk.Label):
 
 
 
-class PasswordLabel(Label):
+class PasswordLabel(EventBox):
 	"A label for displaying passwords"
 
-	def __init__(self, password = "", cfg = None, justify = gtk.JUSTIFY_LEFT):
-		Label.__init__(self, password, justify)
+	def __init__(self, password = "", cfg = None, clipboard = None, justify = gtk.JUSTIFY_LEFT):
+		EventBox.__init__(self)
 
 		self.password	= password
 		self.config	= cfg
-		self.clipboard	= data.Clipboard()
-		self.set_selectable(True)
+		self.clipboard	= clipboard
+
+		self.label = Label(password, justify)
+		self.label.set_selectable(True)
+		self.add(self.label)
 
 		if self.config is not None:
-			self.config.monitor("view/passwords", lambda k,v,d: self.show_password(v))
+			try:
+				self.config.monitor("view/passwords", lambda k,v,d: self.show_password(v))
 
-		self.connect("populate-popup", self.__cb_popup)
+			except config.ConfigError:
+				self.config.monitor("show_passwords", lambda k,v,d: self.show_password(v))
+
+		self.connect("button-press-event", self.__cb_button_press)
+		self.connect("drag-data-get", self.__cb_drag_data_get)
 
 
-	def __cb_popup(self, widget, menu):
+	def __cb_drag_data_get(self, widget, context, selection, info, timestamp, data = None):
+		"Provides data for a drag operation"
+
+		selection.set_text(self.password, -1)
+
+
+	def __cb_button_press(self, widget, data = None):
 		"Populates the popup menu"
 
-		menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
-		menuitem.connect("activate", lambda w: self.clipboard.set(self.password))
+		if self.label.get_selectable() == True:
+			return False
 
-		menu.insert(menuitem, 2)
-		menu.show_all()
+		elif data.button == 3:
+			menu = Menu()
+
+			menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
+			menuitem.connect("activate", lambda w: self.clipboard.set(self.password, True))
+			menu.append(menuitem)
+
+			menu.show_all()
+			menu.popup(None, None, None, data.button, data.time)
+
+			return True
 
 
 	def show_password(self, show = True):
 		"Sets whether to display the password"
 
 		if show == True:
-			self.set_text(self.password)
+			self.label.set_text(self.password)
+			self.label.set_selectable(True)
+			self.drag_source_unset()
 
 		else:
-			self.set_text("******")
+			self.label.set_text("******")
+			self.label.set_selectable(False)
+
+			self.drag_source_set(
+				gtk.gdk.BUTTON1_MASK,
+				(
+					("text/plain", 0, 0),
+					("TEXT", 0, 1),
+					("STRING", 0, 2),
+					("COMPOUND TEXT", 0, 3),
+					("UTF8_STRING", 0, 4),
+				),
+				gtk.gdk.ACTION_COPY
+			)
 
 
 
@@ -628,12 +687,12 @@ gobject.signal_new("changed", FileEntry, gobject.SIGNAL_ACTION, gobject.TYPE_BOO
 class PasswordEntry(Entry):
 	"An entry for editing a password (follows the 'show passwords' preference"
 
-	def __init__(self, cfg = None, password = None):
+	def __init__(self, password = None, cfg = None, clipboard = None):
 		Entry.__init__(self, password)
 		self.set_visibility(False)
 
 		self.config	= cfg
-		self.clipboard	= data.Clipboard()
+		self.clipboard	= clipboard
 
 		if cfg != None:
 			self.config.monitor("view/passwords", lambda k,v,d: self.set_visibility(v))
@@ -668,10 +727,12 @@ class PasswordEntry(Entry):
 	def __cb_popup(self, widget, menu):
 		"Populates the popup menu"
 
-		menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
-		menuitem.connect("activate", lambda w: self.clipboard.set(self.get_text()))
+		if self.clipboard != None:
+			menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
+			menuitem.connect("activate", lambda w: self.clipboard.set(self.get_text(), True))
+
+			menu.insert(menuitem, 2)
 
-		menu.insert(menuitem, 2)
 		menu.show_all()
 
 
@@ -679,11 +740,11 @@ class PasswordEntry(Entry):
 class PasswordEntryGenerate(HBox):
 	"A password entry with a generator button"
 
-	def __init__(self, cfg, password = None):
+	def __init__(self, password = None, cfg = None, clipboard = None):
 		HBox.__init__(self)
 		self.config = cfg
 
-		self.entry = PasswordEntry(cfg, password)
+		self.entry = PasswordEntry(password, cfg, clipboard)
 		self.pack_start(self.entry)
 
 		self.button = Button("Generate", lambda w: self.generate())
@@ -818,7 +879,8 @@ class EntryDropDown(DropDown):
 		DropDown.__init__(self, True)
 
 		for e in entry.ENTRYLIST:
-			self.append_item(e.typename, e.icon, e)
+			if e != entry.FolderEntry:
+				self.append_item(e.typename, e.icon, e)
 
 
 	def get_active_type(self):
@@ -856,7 +918,7 @@ class RadioButton(gtk.RadioButton):
 
 
 
-##### MENU ITEMS #####
+##### MENUS AND MENU ITEMS #####
 
 class ImageMenuItem(gtk.ImageMenuItem):
 	"A menuitem with a stock icon"
@@ -884,6 +946,14 @@ class ImageMenuItem(gtk.ImageMenuItem):
 
 
 
+class Menu(gtk.Menu):
+	"A menu"
+
+	def __init__(self):
+		gtk.Menu.__init__(self)
+
+
+
 ##### MISCELLANEOUS WIDGETS #####
 
 class TreeView(gtk.TreeView):
@@ -1163,7 +1233,8 @@ class ItemFactory(gtk.IconFactory):
 		if config.DIR_ICONS not in self.theme.get_search_path():
 			self.theme.append_search_path(config.DIR_ICONS)
 
-		self.load_stock_icon(STOCK_REVELATION, "revelation", ( ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG ))
+		self.load_stock_icon(STOCK_REVELATION, "revelation", ( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU ))
+		self.load_stock_icon(STOCK_REVELATION_LOCKED, "revelation-locked", ( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU ))
 
 		self.__init_entryicons()
 		self.__init_items()
@@ -1203,15 +1274,17 @@ class ItemFactory(gtk.IconFactory):
 		"Creates stock items"
 
 		items = (
-			( STOCK_ADD,		"_Add Entry",	gtk.STOCK_ADD ),
 			( STOCK_CONTINUE,	"_Continue",	"stock_test-mode" ),
 			( STOCK_DISCARD,	"_Discard",	gtk.STOCK_DELETE ),
 			( STOCK_EDIT,		"_Edit",	"stock_edit" ),
 			( STOCK_EXPORT,		"_Export",	gtk.STOCK_EXECUTE ),
+			( STOCK_FOLDER,		"",		"stock_folder" ),
 			( STOCK_GENERATE,	"_Generate",	gtk.STOCK_EXECUTE ),
 			( STOCK_GOTO,		"_Go to",	gtk.STOCK_JUMP_TO ),
 			( STOCK_IMPORT,		"_Import",	gtk.STOCK_CONVERT ),
 			( STOCK_LOCK,		"_Lock",	"stock_lock" ),
+			( STOCK_NEW_ENTRY,	"_Add Entry",	gtk.STOCK_ADD ),
+			( STOCK_NEW_FOLDER,	"_Add Folder",	"stock_folder" ),
 			( STOCK_NEXT,		"Ne_xt",	gtk.STOCK_GO_FORWARD ),
 			( STOCK_OVERWRITE,	"_Overwrite",	gtk.STOCK_SAVE_AS ),
 			( STOCK_PASSWORD_CHANGE,"_Change",	"stock_lock-ok" ),
@@ -1255,7 +1328,13 @@ class ItemFactory(gtk.IconFactory):
 	def load_icon(self, id, size):
 		"Loads an icon"
 
-		return self.theme.load_icon(id, size, 0)
+		if self.theme.has_icon(id):
+			pixbuf = self.theme.load_icon(id, size, 0)
+
+		else:
+			return None
+
+		return pixbuf
 
 
 	def load_stock_icon(self, id, icon, sizes):
@@ -1271,65 +1350,17 @@ class ItemFactory(gtk.IconFactory):
 				source.set_size(size)
 				source.set_size_wildcarded(False)
 
-				pixbuf = self.theme.load_icon(icon, pixelsize, 0)
-				source.set_pixbuf(pixbuf)
+				pixbuf = self.load_icon(icon, pixelsize)
+
+				if pixbuf != None:
+					source.set_pixbuf(pixbuf)
+
 				iconset.add_source(source)
 
 		self.add(id, iconset)
 
 
 
-class Timer(gobject.GObject):
-	"Handles timeouts etc"
-
-	def __init__(self, resolution = 1):
-		gobject.GObject.__init__(self)
-
-		self.offset		= None
-		self.timeout		= None
-
-		gobject.timeout_add(resolution * 1000, self.__cb_check)
-
-
-	def __cb_check(self):
-		"Checks if the timeout has been reached"
-
-		if None not in (self.offset, self.timeout) and int(time.time()) >= (self.offset + self.timeout):
-			self.stop()
-			self.emit("ring")
-
-		return True
-
-
-	def reset(self):
-		"Resets the timer"
-
-		if self.offset != None:
-			self.offset = int(time.time())
-
-
-	def start(self, timeout):
-		"Starts the timer"
-
-		if timeout == 0:
-			self.stop()
-
-		else:
-			self.offset = int(time.time())
-			self.timeout = timeout
-
-
-	def stop(self):
-		"Stops the timer"
-
-		self.offset = None
-		self.timeout = None
-
-
-gobject.signal_new("ring", Timer, gobject.SIGNAL_ACTION, gobject.TYPE_BOOLEAN, ())
-
-
-
 class UIManager(gtk.UIManager):
 	"UI item manager"
 
@@ -1396,12 +1427,16 @@ class UIManager(gtk.UIManager):
 					"label"		: "",
 					"stock"		: "",
 					"accel"		: None,
-					"description"	: ""
+					"description"	: "",
+					"important"	: False
 				}
 
 				if actionnode.attributes.has_key("type"):
 					actiondata["type"] = actionnode.attributes["type"].nodeValue
 
+				if actionnode.attributes.has_key("important"):
+					actiondata["important"] = (actionnode.attributes["important"].nodeValue == "yes")
+
 				for node in actionnode.childNodes:
 
 					if node.nodeType != node.ELEMENT_NODE:
@@ -1431,6 +1466,7 @@ class UIManager(gtk.UIManager):
 				else:
 					raise DataError
 
+				action.set_property("is-important", actiondata["important"])
 				actiongroup.add_action_with_accel(action, actiondata["accel"])
 
 
@@ -1583,16 +1619,18 @@ class App(gnome.ui.App):
 class EntryView(VBox):
 	"A component for displaying an entry"
 
-	def __init__(self, cfg = None):
+	def __init__(self, cfg = None, clipboard = None):
 		VBox.__init__(self)
-		self.config = cfg
 		self.set_spacing(15)
 		self.set_border_width(10)
 
-		self.size_name	= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-		self.size_value	= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+		self.config		= cfg
+		self.clipboard		= clipboard
 
-		self.entry = None
+		self.size_name		= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+		self.size_value		= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+
+		self.entry		= None
 
 
 	def clear(self, force = False):
@@ -1643,7 +1681,7 @@ class EntryView(VBox):
 				self.size_name.add_widget(label)
 				row.pack_start(label, False, False)
 
-				widget = generate_field_display_widget(field, self.config)
+				widget = generate_field_display_widget(field, self.config, self.clipboard)
 				self.size_value.add_widget(widget)
 				row.pack_start(widget, False, False)
 
